package org.test;

import javax.servlet.annotation.WebServlet;

import com.vaadin.annotations.Theme;
import com.vaadin.annotations.VaadinServletConfiguration;
import com.vaadin.server.*;
import com.vaadin.shared.ui.dnd.DropEffect;
import com.vaadin.shared.ui.dnd.EffectAllowed;
import com.vaadin.ui.*;
import com.vaadin.ui.dnd.DragSourceExtension;
import com.vaadin.ui.dnd.DropTargetExtension;
import com.vaadin.ui.themes.ValoTheme;

import java.util.Optional;

/**
 * Demonstrates Drag and Drop bug in Vaadin 8.1.14
 * Bug happening with Firefox version 109, but not
 * with earlier versions (no problem when using Firefox 108.0.2)
 *
 * Layout program a Label drop target and three sources:
 * - Label
 * - Image with src
 * - Image with missing src
 *
 *  It will be seen that dragging the regular image to the target does not generate
 *  a drop event. However, drop event IS generated as expected when dragging the label
 *  or the image with missing src.*
 */
@Theme("mytheme")
public class MyUI extends UI {

    private TextArea logMsgs;

    private void logMsg(String msg) {
        logMsgs.setValue( logMsgs.getValue() + "\n" + msg );
    }

    @Override
    protected void init(VaadinRequest vaadinRequest) {
        final VerticalLayout layout = new VerticalLayout();
        
        layout.setSpacing(true);

        Label target = new Label("LABEL TARGET");
        target.setHeight("100px");
        target.setWidth("200px");

        VerticalLayout targetContainer = new VerticalLayout();
        targetContainer.setCaption("Vbox drop target");
        targetContainer.setWidth("200px");
        targetContainer.setHeight("200px");
        targetContainer.addStyleName(ValoTheme.LAYOUT_CARD);

        Label source = new Label("LABEL SOURCE");
        source.setId("labelSource");
        target.setHeight("100px");
        target.setWidth("200px");

        // Serve the image from the theme
//        Resource res = new ThemeResource("app-icon-menu.png");
        Resource res = new ExternalResource("https://f.vividscreen.info/soft/4b6a5b4b7a9efe46e38faac9fa52aa0e/Pembroke-Welsh-Corgi-320x240.jpg");
        Image sourceImg = new Image("IMAGE SOURCE", res);
        sourceImg.setId("imageId");
        sourceImg.setWidth("200px");
        sourceImg.setHeight("100px");


        Image sourceImg2 = new Image("IMAGE SOURCE", null);
        sourceImg2.setId("noImageId");
        sourceImg2.setWidth("200px");
        sourceImg2.setHeight("100px");


        DragSourceExtension<Label> iSource = new DragSourceExtension<>(source);
        iSource.setEffectAllowed(EffectAllowed.MOVE);

        DragSourceExtension<Image> iSource2 = new DragSourceExtension<>(sourceImg);
        iSource2.setEffectAllowed(EffectAllowed.MOVE);

        DragSourceExtension<Image> iSource3 = new DragSourceExtension<>(sourceImg2);
        iSource3.setEffectAllowed(EffectAllowed.MOVE);

        iSource.addDragEndListener(event -> {
            logMsg("Label dragEnd " + (event.isCanceled() ? "canceled" : "finished") );
        });

        iSource2.addDragEndListener(event -> {
            logMsg("Filled image dragEnd " + (event.isCanceled() ? "canceled" : "finished") );
        });

        iSource3.addDragEndListener(event -> {
            logMsg("Blank image dragEnd " + (event.isCanceled() ? "canceled" : "finished") );
        });

        DropTargetExtension<VerticalLayout> dropTarget = new DropTargetExtension<>(targetContainer);

        // the drop effect must match the allowed effect in the drag source for a successful drop
        dropTarget.setDropEffect(DropEffect.MOVE);

        // Insert javascript in the drop criteria to show in the console log
        // that the browser is generating a drop event in all cases, but Vaadin is not calling back to the
        // server with the drop event in the problem case
        dropTarget.setDropCriteriaScript("if (event.type == 'drop') { console.log(\"Event obj: %o\", event ); } return true;");

        // Display a msg in the log textarea when the drop event is triggered
        dropTarget.addDropListener(dropEvent -> {
            AbstractComponent draggedComponent = dropEvent.getDragSourceComponent().orElse(null);
            logMsg("DropEvent dragged component id:" + (draggedComponent == null ? null : draggedComponent.getId()));
        } );

        // Create a text area to show event activity
        logMsgs = new TextArea("Events");
        logMsgs.setWidth("100%");
        logMsgs.setHeight("10em");

        layout.addComponents(target, targetContainer, source, sourceImg, sourceImg2, logMsgs);

        setContent(layout);

    }

    @WebServlet(urlPatterns = "/*", name = "MyUIServlet", asyncSupported = true)
    @VaadinServletConfiguration(ui = MyUI.class, productionMode = false)
    public static class MyUIServlet extends VaadinServlet {
    }
}
